import json
import pathlib
import jwt
from datetime import datetime, timedelta
from pathlib import Path
import requests
from netsuite.NetsuiteToken import NetsuiteToken
from netsuite.swagger_client.rest_client import RestClient, QueryClient
from netsuite.settings import APISettings
from netsuite.storages import BaseStorage, JSONStorage


class Netsuite:
    app_name: str = None
    storage: BaseStorage = None
    api_settings: APISettings
    rest_client = None
    query_client = None

    def __init__(self, config: dict = None, config_file: Path = None):
        if config and config_file:
            raise ValueError("You can only load settings from one source")
        if config_file:
            with open(config_file, 'r') as f:
                config = json.load(f)
        if config is None and config_file is None:
            try:
                with open(pathlib.Path(APISettings().defaults.get("CREDENTIALS_PATH")), 'r') as f:
                    config = json.load(f)
            except Exception as e:
                raise Exception("No Configuration Present. Try Generating one.")

        self.api_settings = APISettings(config)
        if not self.api_settings.CLIENT_ID:
            raise Exception("Missing Client Id")
        if not self.api_settings.NETSUITE_APP_NAME:
            raise Exception("Missing Netsuite App Name")
        if not self.api_settings.NETSUITE_KEY_FILE:
            raise Exception("Missing Netsuite Certificate path.")
        if not self.api_settings.CERT_ID:
            raise Exception("Missing Netsuite Certificate ID.")

        self.app_name = self.api_settings.APP_NAME
        self.netsuite_app_name = self.api_settings.NETSUITE_APP_NAME
        self.netsuite_key_path = self.api_settings.NETSUITE_KEY_FILE
        self.netsuite_cert_id = self.api_settings.CERT_ID

        self.storage = self.api_settings.STORAGE_CLASS()
        if isinstance(self.api_settings.STORAGE_CLASS(), JSONStorage):
            if not self.api_settings.JSON_STORAGE_PATH:
                raise Exception("JSON_STORAGE_PATH must be defined for JSONStorage")
            self.storage.storage_path = self.api_settings.JSON_STORAGE_PATH
        self.rest_url = f"https://{self.api_settings.NETSUITE_APP_NAME}.suitetalk.api.netsuite.com/services/rest" \
                        f"/record/v1/ "
        self.access_token_url = f"https://{self.api_settings.NETSUITE_APP_NAME}.suitetalk.api.netsuite.com/services/rest/auth/oauth2/v1/token",
        self.status_dict = None
        self.categories = None

    @property
    def REST_CLIENT(self):
        if not self.rest_client:
            self.rest_client = RestClient(self)
            if self.token.access_token is not None:
                self.get_customer_categories()
                self.get_status_dict()
        return self.rest_client

    @property
    def QUERY_CLIENT(self):
        if not self.query_client:
            self.query_client = QueryClient(self)
            if self.token.access_token is not None:
                self.get_customer_categories()
                self.get_status_dict()
        return self.query_client

    @property
    def token(self) -> NetsuiteToken:
        return self.storage.get_token(self.app_name)

    def save_token(self, token):
        self.storage.save_token(self.app_name, token)

    def get_jwt(self):
        private_key = ""
        with open(self.netsuite_key_path, "rb") as pemfile:
            private_key = pemfile.read()
        payload = {
            "iss": f"{self.api_settings.CLIENT_ID}",
            "scope": "rest_webservices",
            "aud": f"{self.access_token_url}",
            "exp": (datetime.now() + timedelta(seconds=3600)).timestamp(),
            "iat": datetime.now().timestamp()
        }

        headers = {
            "typ": "JWT",
            "alg": "RS256",
            "kid": f"{self.netsuite_cert_id}"
        }
        jwt_token = jwt.encode(payload=payload, key=private_key, algorithm='RS256', headers=headers)

        return jwt_token

    def request_access_token(self):
        json_web_token = self.get_jwt()
        data = {
            'grant_type': 'client_credentials',
            'client_assertion_type': 'urn:ietf:params:oauth:client-assertion-type:jwt-bearer',
            'client_assertion': f'{json_web_token}'
        }

        headers = {
            'Content-Type': 'application/x-www-form-urlencoded'
        }

        url = f"https://{self.api_settings.NETSUITE_APP_NAME}.suitetalk.api.netsuite.com/services/rest/auth/oauth2/v1/token"
        response = requests.post(url, data=data, headers=headers)
        token = NetsuiteToken(**response.json())
        self.save_token(token)
        return self.token

    def change_app(self, app_name):
        self.app_name = app_name
        if not self.token.access_token:
            raise Exception(f"{app_name} does not have a token in storage, please authenticate")
        # self.rest_v1 = REST_V1(self)
        self.rest_client = RestClient(self)

    def get_token(self):
        if not self.token.is_expired:
            return self.token
        else:
            return self.request_access_token()

    def get_status_dict(self):
        if self.token.access_token is None:
            return None
        if self.status_dict is None:
            query = "SELECT * FROM EntityStatus WHERE inactive = 'F'"
            statuses = self.QUERY_CLIENT.query_api.execute_query(query=query)
            status_dict = {}
            for status in statuses:
                status_dict[f"{status.get('entitytype').upper()}-{status.get('name').upper()}"] = status.get('key')
            self.status_dict = status_dict

        return self.status_dict

    def get_customer_categories(self):
        if self.token.access_token is None:
            return None
        if self.categories is None:
            query = "SELECT * FROM customercategory WHERE isinactive = 'F'"
            categories = self.QUERY_CLIENT.query_api.execute_query(query=query)
            category_dict = {}
            for category in categories:
                category_dict[f"{category.get('name').upper()}"] = category.get('id')
                self.categories = category_dict
        return self.categories



from netsuite.Netsuite import NetsuiteToken


class BaseStorage:
    app: str = None

    def __init__(self):
        pass

    def get_token(self, app) -> NetsuiteToken:
        raise NotImplementedError()

    def save_token(self, app: str, token: NetsuiteToken) -> bool:
        raise NotImplementedError()



# coding: utf-8

"""
    NetSuite REST API

    NetSuite REST Record API generated 2023-01-18 at 19:40:11 UTC for account 472052, user will@theapiguys.com with role Keap Integration.  # noqa: E501

    OpenAPI spec version: v1
    Contact: info@netsuite.com
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""

import pprint
import re  # noqa: F401

import six

class CustomeraddressBookaddressBookAddressCountry(object):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """
    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {
        'id': 'str',
        'ref_name': 'str'
    }

    attribute_map = {
        'id': 'id',
        'ref_name': 'refName'
    }

    def __init__(self, id=None, ref_name=None):  # noqa: E501
        """CustomeraddressBookaddressBookAddressCountry - a model defined in Swagger"""  # noqa: E501
        self._id = None
        self._ref_name = None
        self.discriminator = None
        if id is not None:
            self.id = id
        if ref_name is not None:
            self.ref_name = ref_name

    @property
    def id(self):
        """Gets the id of this CustomeraddressBookaddressBookAddressCountry.  # noqa: E501


        :return: The id of this CustomeraddressBookaddressBookAddressCountry.  # noqa: E501
        :rtype: str
        """
        return self._id

    @id.setter
    def id(self, id):
        """Sets the id of this CustomeraddressBookaddressBookAddressCountry.


        :param id: The id of this CustomeraddressBookaddressBookAddressCountry.  # noqa: E501
        :type: str
        """
        allowed_values = ["PR", "PS", "PT", "PW", "PY", "QA", "AD", "AE", "AF", "AG", "AI", "AL", "AM", "AN", "AO", "AQ", "AR", "AS", "AT", "RE", "AU", "AW", "AX", "AZ", "RO", "BA", "BB", "RS", "BD", "BE", "RU", "BF", "BG", "RW", "BH", "BI", "BJ", "BL", "BM", "BN", "BO", "SA", "SB", "BQ", "BR", "SC", "SD", "BS", "SE", "BT", "BV", "SG", "SH", "BW", "SI", "SJ", "BY", "BZ", "SK", "SL", "SM", "SN", "SO", "CA", "SR", "SS", "CC", "CD", "ST", "CF", "SV", "CG", "CH", "SX", "SY", "CI", "SZ", "CK", "CL", "CM", "CN", "CO", "TC", "CR", "TD", "CS", "TF", "CU", "TG", "CV", "CW", "TH", "CX", "TJ", "CY", "TK", "CZ", "TL", "TM", "TN", "TO", "TR", "TT", "DE", "TV", "TW", "TZ", "DJ", "DK", "DM", "DO", "UA", "UG", "DZ", "UM", "EA", "US", "EC", "EE", "EG", "EH", "UY", "UZ", "VA", "VC", "ER", "ES", "VE", "ET", "VG", "VI", "VN", "VU", "FI", "FJ", "FK", "FM", "FO", "FR", "WF", "GA", "GB", "WS", "GD", "GE", "GF", "GG", "GH", "GI", "GL", "GM", "GN", "GP", "GQ", "GR", "GS", "GT", "GU", "GW", "GY", "XK", "HK", "HM", "HN", "HR", "YE", "HT", "HU", "IC", "ID", "YT", "IE", "IL", "IM", "IN", "IO", "ZA", "IQ", "IR", "IS", "IT", "ZM", "JE", "ZW", "JM", "JO", "JP", "KE", "KG", "KH", "KI", "KM", "KN", "KP", "KR", "KW", "KY", "KZ", "LA", "LB", "LC", "LI", "LK", "LR", "LS", "LT", "LU", "LV", "LY", "MA", "MC", "MD", "ME", "MF", "MG", "MH", "MK", "ML", "MM", "MN", "MO", "MP", "MQ", "MR", "MS", "MT", "MU", "MV", "MW", "MX", "MY", "MZ", "NA", "NC", "NE", "NF", "NG", "NI", "NL", "NO", "NP", "NR", "NU", "NZ", "OM", "PA", "PE", "PF", "PG", "PH", "PK", "PL", "PM", "PN"]  # noqa: E501
        if id not in allowed_values:
            raise ValueError(
                "Invalid value for `id` ({0}), must be one of {1}"  # noqa: E501
                .format(id, allowed_values)
            )

        self._id = id

    @property
    def ref_name(self):
        """Gets the ref_name of this CustomeraddressBookaddressBookAddressCountry.  # noqa: E501


        :return: The ref_name of this CustomeraddressBookaddressBookAddressCountry.  # noqa: E501
        :rtype: str
        """
        return self._ref_name

    @ref_name.setter
    def ref_name(self, ref_name):
        """Sets the ref_name of this CustomeraddressBookaddressBookAddressCountry.


        :param ref_name: The ref_name of this CustomeraddressBookaddressBookAddressCountry.  # noqa: E501
        :type: str
        """

        self._ref_name = ref_name

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value
        if issubclass(CustomeraddressBookaddressBookAddressCountry, dict):
            for key, value in self.items():
                result[key] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, CustomeraddressBookaddressBookAddressCountry):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other



# coding: utf-8

"""
    NetSuite REST API

    NetSuite REST Record API generated 2023-01-18 at 19:40:11 UTC for account 472052, user will@theapiguys.com with role Keap Integration.  # noqa: E501

    OpenAPI spec version: v1
    Contact: info@netsuite.com
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""

import pprint
import re  # noqa: F401

import six

class CustomerCampaignsCollection(object):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """
    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {
        'links': 'list[NsLink]',
        'total_results': 'int',
        'count': 'int',
        'has_more': 'bool',
        'offset': 'int',
        'items': 'list[CustomerCampaignsElement]'
    }

    attribute_map = {
        'links': 'links',
        'total_results': 'totalResults',
        'count': 'count',
        'has_more': 'hasMore',
        'offset': 'offset',
        'items': 'items'
    }

    def __init__(self, links=None, total_results=None, count=None, has_more=None, offset=None, items=None):  # noqa: E501
        """CustomerCampaignsCollection - a model defined in Swagger"""  # noqa: E501
        self._links = None
        self._total_results = None
        self._count = None
        self._has_more = None
        self._offset = None
        self._items = None
        self.discriminator = None
        if links is not None:
            self.links = links
        if total_results is not None:
            self.total_results = total_results
        if count is not None:
            self.count = count
        if has_more is not None:
            self.has_more = has_more
        if offset is not None:
            self.offset = offset
        if items is not None:
            self.items = items

    @property
    def links(self):
        """Gets the links of this CustomerCampaignsCollection.  # noqa: E501


        :return: The links of this CustomerCampaignsCollection.  # noqa: E501
        :rtype: list[NsLink]
        """
        return self._links

    @links.setter
    def links(self, links):
        """Sets the links of this CustomerCampaignsCollection.


        :param links: The links of this CustomerCampaignsCollection.  # noqa: E501
        :type: list[NsLink]
        """

        self._links = links

    @property
    def total_results(self):
        """Gets the total_results of this CustomerCampaignsCollection.  # noqa: E501


        :return: The total_results of this CustomerCampaignsCollection.  # noqa: E501
        :rtype: int
        """
        return self._total_results

    @total_results.setter
    def total_results(self, total_results):
        """Sets the total_results of this CustomerCampaignsCollection.


        :param total_results: The total_results of this CustomerCampaignsCollection.  # noqa: E501
        :type: int
        """

        self._total_results = total_results

    @property
    def count(self):
        """Gets the count of this CustomerCampaignsCollection.  # noqa: E501


        :return: The count of this CustomerCampaignsCollection.  # noqa: E501
        :rtype: int
        """
        return self._count

    @count.setter
    def count(self, count):
        """Sets the count of this CustomerCampaignsCollection.


        :param count: The count of this CustomerCampaignsCollection.  # noqa: E501
        :type: int
        """

        self._count = count

    @property
    def has_more(self):
        """Gets the has_more of this CustomerCampaignsCollection.  # noqa: E501


        :return: The has_more of this CustomerCampaignsCollection.  # noqa: E501
        :rtype: bool
        """
        return self._has_more

    @has_more.setter
    def has_more(self, has_more):
        """Sets the has_more of this CustomerCampaignsCollection.


        :param has_more: The has_more of this CustomerCampaignsCollection.  # noqa: E501
        :type: bool
        """

        self._has_more = has_more

    @property
    def offset(self):
        """Gets the offset of this CustomerCampaignsCollection.  # noqa: E501


        :return: The offset of this CustomerCampaignsCollection.  # noqa: E501
        :rtype: int
        """
        return self._offset

    @offset.setter
    def offset(self, offset):
        """Sets the offset of this CustomerCampaignsCollection.


        :param offset: The offset of this CustomerCampaignsCollection.  # noqa: E501
        :type: int
        """

        self._offset = offset

    @property
    def items(self):
        """Gets the items of this CustomerCampaignsCollection.  # noqa: E501


        :return: The items of this CustomerCampaignsCollection.  # noqa: E501
        :rtype: list[CustomerCampaignsElement]
        """
        return self._items

    @items.setter
    def items(self, items):
        """Sets the items of this CustomerCampaignsCollection.


        :param items: The items of this CustomerCampaignsCollection.  # noqa: E501
        :type: list[CustomerCampaignsElement]
        """

        self._items = items

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value
        if issubclass(CustomerCampaignsCollection, dict):
            for key, value in self.items():
                result[key] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, CustomerCampaignsCollection):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other



from time import time


class NetsuiteToken:
    def __init__(self, **kwargs):
        self.access_token = kwargs.get("access_token", None)
        self.refresh_token = kwargs.get("refresh_token", None)
        self.expires_in = kwargs.get("expires_in", 0)
        self.end_of_life = kwargs.get("end_of_life", int(time()) + int(self.expires_in))
        self.scope = kwargs.get("scope", None)

    @property
    def is_expired(self):
        return self.end_of_life < int(time())



from setuptools import setup, find_packages
from pathlib import Path
this_directory = Path(__file__).parent
long_description = (this_directory / "README.md").read_text()

setup(
    name='netsuite_python',
    version='1.1.8',
    description='Python SDK for Netsuite API with Django Integration',
    long_description=long_description,
    long_description_content_type='text/markdown',
    url='https://bitbucket.org/theapiguys/netsuite_python',
    readme="README.md",
    author='Will @ TheAPIGuys',
    author_email='will@theapiguys.com',
    packages=find_packages(),
    include_package_data=True,
    install_requires=[
        'Click',
        'requests',
        'PyJWT',
        "urllib3 >= 1.15",
        "six >= 1.10",
        "certifi",
        "python-dateutil",
        "pyOpenSSL"
    ],
    entry_points={
        'console_scripts': [
            'netsuite = netsuite.scripts.cli:cli',
        ],

    },
)



# coding: utf-8

"""
    NetSuite REST API

    NetSuite REST Record API generated 2023-01-18 at 19:40:11 UTC for account 472052, user will@theapiguys.com with role Keap Integration.  # noqa: E501

    OpenAPI spec version: v1
    Contact: info@netsuite.com
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""

import pprint
import re  # noqa: F401

import six

class CustomerAddressBookAddressBookAddress(object):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """
    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {
        'zip': 'str',
        'country': 'CustomeraddressBookaddressBookAddressCountry',
        'addr2': 'str',
        'city': 'str',
        'addr1': 'str',
        'last_modified_date': 'datetime',
        'addr3': 'str',
        'external_id': 'str',
        'custrecordpca_search': 'str',
        'addr_phone': 'str',
        'addressee': 'str',
        'attention': 'str',
        'override': 'bool',
        'state': 'str',
        'addr_text': 'str',
        'ref_name': 'str',
        'links': 'list[NsLink]'
    }

    attribute_map = {
        'zip': 'zip',
        'country': 'country',
        'addr2': 'addr2',
        'city': 'city',
        'addr1': 'addr1',
        'last_modified_date': 'lastModifiedDate',
        'addr3': 'addr3',
        'external_id': 'externalId',
        'custrecordpca_search': 'custrecordpca_search',
        'addr_phone': 'addrPhone',
        'addressee': 'addressee',
        'attention': 'attention',
        'override': 'override',
        'state': 'state',
        'addr_text': 'addrText',
        'ref_name': 'refName',
        'links': 'links'
    }

    def __init__(self, zip=None, country=None, addr2=None, city=None, addr1=None, last_modified_date=None, addr3=None, external_id=None, custrecordpca_search=None, addr_phone=None, addressee=None, attention=None, override=None, state=None, addr_text=None, ref_name=None, links=None):  # noqa: E501
        """CustomerAddressBookAddressBookAddress - a model defined in Swagger"""  # noqa: E501
        self._zip = None
        self._country = None
        self._addr2 = None
        self._city = None
        self._addr1 = None
        self._last_modified_date = None
        self._addr3 = None
        self._external_id = None
        self._custrecordpca_search = None
        self._addr_phone = None
        self._addressee = None
        self._attention = None
        self._override = None
        self._state = None
        self._addr_text = None
        self._ref_name = None
        self._links = None
        self.discriminator = None
        if zip is not None:
            self.zip = zip
        if country is not None:
            self.country = country
        if addr2 is not None:
            self.addr2 = addr2
        if city is not None:
            self.city = city
        if addr1 is not None:
            self.addr1 = addr1
        if last_modified_date is not None:
            self.last_modified_date = last_modified_date
        if addr3 is not None:
            self.addr3 = addr3
        if external_id is not None:
            self.external_id = external_id
        if custrecordpca_search is not None:
            self.custrecordpca_search = custrecordpca_search
        if addr_phone is not None:
            self.addr_phone = addr_phone
        if addressee is not None:
            self.addressee = addressee
        if attention is not None:
            self.attention = attention
        if override is not None:
            self.override = override
        if state is not None:
            self.state = state
        if addr_text is not None:
            self.addr_text = addr_text
        if ref_name is not None:
            self.ref_name = ref_name
        if links is not None:
            self.links = links

    @property
    def zip(self):
        """Gets the zip of this CustomerAddressBookAddressBookAddress.  # noqa: E501


        :return: The zip of this CustomerAddressBookAddressBookAddress.  # noqa: E501
        :rtype: str
        """
        return self._zip

    @zip.setter
    def zip(self, zip):
        """Sets the zip of this CustomerAddressBookAddressBookAddress.


        :param zip: The zip of this CustomerAddressBookAddressBookAddress.  # noqa: E501
        :type: str
        """

        self._zip = zip

    @property
    def country(self):
        """Gets the country of this CustomerAddressBookAddressBookAddress.  # noqa: E501


        :return: The country of this CustomerAddressBookAddressBookAddress.  # noqa: E501
        :rtype: CustomeraddressBookaddressBookAddressCountry
        """
        return self._country

    @country.setter
    def country(self, country):
        """Sets the country of this CustomerAddressBookAddressBookAddress.


        :param country: The country of this CustomerAddressBookAddressBookAddress.  # noqa: E501
        :type: CustomeraddressBookaddressBookAddressCountry
        """

        self._country = country

    @property
    def addr2(self):
        """Gets the addr2 of this CustomerAddressBookAddressBookAddress.  # noqa: E501


        :return: The addr2 of this CustomerAddressBookAddressBookAddress.  # noqa: E501
        :rtype: str
        """
        return self._addr2

    @addr2.setter
    def addr2(self, addr2):
        """Sets the addr2 of this CustomerAddressBookAddressBookAddress.


        :param addr2: The addr2 of this CustomerAddressBookAddressBookAddress.  # noqa: E501
        :type: str
        """

        self._addr2 = addr2

    @property
    def city(self):
        """Gets the city of this CustomerAddressBookAddressBookAddress.  # noqa: E501


        :return: The city of this CustomerAddressBookAddressBookAddress.  # noqa: E501
        :rtype: str
        """
        return self._city

    @city.setter
    def city(self, city):
        """Sets the city of this CustomerAddressBookAddressBookAddress.


        :param city: The city of this CustomerAddressBookAddressBookAddress.  # noqa: E501
        :type: str
        """

        self._city = city

    @property
    def addr1(self):
        """Gets the addr1 of this CustomerAddressBookAddressBookAddress.  # noqa: E501


        :return: The addr1 of this CustomerAddressBookAddressBookAddress.  # noqa: E501
        :rtype: str
        """
        return self._addr1

    @addr1.setter
    def addr1(self, addr1):
        """Sets the addr1 of this CustomerAddressBookAddressBookAddress.


        :param addr1: The addr1 of this CustomerAddressBookAddressBookAddress.  # noqa: E501
        :type: str
        """

        self._addr1 = addr1

    @property
    def last_modified_date(self):
        """Gets the last_modified_date of this CustomerAddressBookAddressBookAddress.  # noqa: E501


        :return: The last_modified_date of this CustomerAddressBookAddressBookAddress.  # noqa: E501
        :rtype: datetime
        """
        return self._last_modified_date

    @last_modified_date.setter
    def last_modified_date(self, last_modified_date):
        """Sets the last_modified_date of this CustomerAddressBookAddressBookAddress.


        :param last_modified_date: The last_modified_date of this CustomerAddressBookAddressBookAddress.  # noqa: E501
        :type: datetime
        """

        self._last_modified_date = last_modified_date

    @property
    def addr3(self):
        """Gets the addr3 of this CustomerAddressBookAddressBookAddress.  # noqa: E501


        :return: The addr3 of this CustomerAddressBookAddressBookAddress.  # noqa: E501
        :rtype: str
        """
        return self._addr3

    @addr3.setter
    def addr3(self, addr3):
        """Sets the addr3 of this CustomerAddressBookAddressBookAddress.


        :param addr3: The addr3 of this CustomerAddressBookAddressBookAddress.  # noqa: E501
        :type: str
        """

        self._addr3 = addr3

    @property
    def external_id(self):
        """Gets the external_id of this CustomerAddressBookAddressBookAddress.  # noqa: E501


        :return: The external_id of this CustomerAddressBookAddressBookAddress.  # noqa: E501
        :rtype: str
        """
        return self._external_id

    @external_id.setter
    def external_id(self, external_id):
        """Sets the external_id of this CustomerAddressBookAddressBookAddress.


        :param external_id: The external_id of this CustomerAddressBookAddressBookAddress.  # noqa: E501
        :type: str
        """

        self._external_id = external_id

    @property
    def custrecordpca_search(self):
        """Gets the custrecordpca_search of this CustomerAddressBookAddressBookAddress.  # noqa: E501


        :return: The custrecordpca_search of this CustomerAddressBookAddressBookAddress.  # noqa: E501
        :rtype: str
        """
        return self._custrecordpca_search

    @custrecordpca_search.setter
    def custrecordpca_search(self, custrecordpca_search):
        """Sets the custrecordpca_search of this CustomerAddressBookAddressBookAddress.


        :param custrecordpca_search: The custrecordpca_search of this CustomerAddressBookAddressBookAddress.  # noqa: E501
        :type: str
        """

        self._custrecordpca_search = custrecordpca_search

    @property
    def addr_phone(self):
        """Gets the addr_phone of this CustomerAddressBookAddressBookAddress.  # noqa: E501


        :return: The addr_phone of this CustomerAddressBookAddressBookAddress.  # noqa: E501
        :rtype: str
        """
        return self._addr_phone

    @addr_phone.setter
    def addr_phone(self, addr_phone):
        """Sets the addr_phone of this CustomerAddressBookAddressBookAddress.


        :param addr_phone: The addr_phone of this CustomerAddressBookAddressBookAddress.  # noqa: E501
        :type: str
        """

        self._addr_phone = addr_phone

    @property
    def addressee(self):
        """Gets the addressee of this CustomerAddressBookAddressBookAddress.  # noqa: E501


        :return: The addressee of this CustomerAddressBookAddressBookAddress.  # noqa: E501
        :rtype: str
        """
        return self._addressee

    @addressee.setter
    def addressee(self, addressee):
        """Sets the addressee of this CustomerAddressBookAddressBookAddress.


        :param addressee: The addressee of this CustomerAddressBookAddressBookAddress.  # noqa: E501
        :type: str
        """

        self._addressee = addressee

    @property
    def attention(self):
        """Gets the attention of this CustomerAddressBookAddressBookAddress.  # noqa: E501


        :return: The attention of this CustomerAddressBookAddressBookAddress.  # noqa: E501
        :rtype: str
        """
        return self._attention

    @attention.setter
    def attention(self, attention):
        """Sets the attention of this CustomerAddressBookAddressBookAddress.


        :param attention: The attention of this CustomerAddressBookAddressBookAddress.  # noqa: E501
        :type: str
        """

        self._attention = attention

    @property
    def override(self):
        """Gets the override of this CustomerAddressBookAddressBookAddress.  # noqa: E501


        :return: The override of this CustomerAddressBookAddressBookAddress.  # noqa: E501
        :rtype: bool
        """
        return self._override

    @override.setter
    def override(self, override):
        """Sets the override of this CustomerAddressBookAddressBookAddress.


        :param override: The override of this CustomerAddressBookAddressBookAddress.  # noqa: E501
        :type: bool
        """

        self._override = override

    @property
    def state(self):
        """Gets the state of this CustomerAddressBookAddressBookAddress.  # noqa: E501


        :return: The state of this CustomerAddressBookAddressBookAddress.  # noqa: E501
        :rtype: str
        """
        return self._state

    @state.setter
    def state(self, state):
        """Sets the state of this CustomerAddressBookAddressBookAddress.


        :param state: The state of this CustomerAddressBookAddressBookAddress.  # noqa: E501
        :type: str
        """

        self._state = state

    @property
    def addr_text(self):
        """Gets the addr_text of this CustomerAddressBookAddressBookAddress.  # noqa: E501


        :return: The addr_text of this CustomerAddressBookAddressBookAddress.  # noqa: E501
        :rtype: str
        """
        return self._addr_text

    @addr_text.setter
    def addr_text(self, addr_text):
        """Sets the addr_text of this CustomerAddressBookAddressBookAddress.


        :param addr_text: The addr_text of this CustomerAddressBookAddressBookAddress.  # noqa: E501
        :type: str
        """

        self._addr_text = addr_text

    @property
    def ref_name(self):
        """Gets the ref_name of this CustomerAddressBookAddressBookAddress.  # noqa: E501


        :return: The ref_name of this CustomerAddressBookAddressBookAddress.  # noqa: E501
        :rtype: str
        """
        return self._ref_name

    @ref_name.setter
    def ref_name(self, ref_name):
        """Sets the ref_name of this CustomerAddressBookAddressBookAddress.


        :param ref_name: The ref_name of this CustomerAddressBookAddressBookAddress.  # noqa: E501
        :type: str
        """

        self._ref_name = ref_name

    @property
    def links(self):
        """Gets the links of this CustomerAddressBookAddressBookAddress.  # noqa: E501


        :return: The links of this CustomerAddressBookAddressBookAddress.  # noqa: E501
        :rtype: list[NsLink]
        """
        return self._links

    @links.setter
    def links(self, links):
        """Sets the links of this CustomerAddressBookAddressBookAddress.


        :param links: The links of this CustomerAddressBookAddressBookAddress.  # noqa: E501
        :type: list[NsLink]
        """

        self._links = links

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value
        if issubclass(CustomerAddressBookAddressBookAddress, dict):
            for key, value in self.items():
                result[key] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, CustomerAddressBookAddressBookAddress):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other



# coding: utf-8

"""
    NetSuite REST API
"""

from __future__ import absolute_import

import re  # noqa: F401

# python 2 and python 3 compatibility library
import six
import json

from netsuite.swagger_client.api_client import ApiClient
import netsuite.swagger_client.models


class QueryApi(object):

    def __init__(self, api_client=None):
        if api_client is None:
            api_client = ApiClient()
        self.api_client = api_client

    def execute_query(self, query, **kwargs):
        all_params = ['prefer', 'response_type']  # noqa: E501
        params = locals()
        for key, val in six.iteritems(params['kwargs']):
            if key not in all_params:
                raise TypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method customer_get" % key
                )
            params[key] = val
        del params['kwargs']

        header_params = {}
        if 'prefer' in params:
            header_params['Prefer'] = params['prefer']  # noqa: E501
        else:
            header_params['Prefer'] = 'transient'

        if 'response_type' in params:
            if params['response_type'] is not None:
                if hasattr(netsuite.swagger_client.models, params['response_type']):
                    response_type = params['response_type']
                else:
                    response_type = None
                    raise TypeError(f"{params['response_type']} is not a valid response type")
            return_http_only_data = True
            _preload_content = True
            # print(params['response_type'])
        else:
            response_type = None
            return_http_only_data = True
            _preload_content = False

        # Authentication setting
        auth_settings = ['oAuth2ClientCredentials']
        response = self.api_client.call_api('',
                                            'POST',
                                            header_params=header_params,
                                            auth_settings=auth_settings,
                                            body={"q": f"{query}"},
                                            response_type=response_type,
                                            _return_http_data_only=return_http_only_data,
                                            _preload_content=_preload_content)

        if response_type is not None:
            if hasattr(response, 'items'):
                return response.items
            else:
                return response
        if isinstance(response, tuple):
            return response
        if hasattr(response, 'data'):
            if isinstance(response.data, bytes):
                response = json.loads(response.data.decode('UTF-8'))
                if 'items' in response:
                    if type(response.get("items")) is list:
                        return response.get("items")
                    else:
                        items = [response.get("items")]
                        return items
                else:
                    return None
        else:
            return response



# coding: utf-8

"""
    NetSuite REST Record API

    NetSuite REST Record API generated 2023-02-11 at 17:13:58 UTC for account 472052_SB1, user will@theapiguys.com with role Keap Integration.  # noqa: E501

    OpenAPI spec version: v1
    Contact: info@netsuite.com
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""

from __future__ import absolute_import

import re  # noqa: F401

# python 2 and python 3 compatibility library
import six

from netsuite.swagger_client.api_client import ApiClient


class MessageApi(object):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    Ref: https://github.com/swagger-api/swagger-codegen
    """

    def __init__(self, api_client=None):
        if api_client is None:
            api_client = ApiClient()
        self.api_client = api_client

    def message_get(self, **kwargs):  # noqa: E501
        """Get list of records.  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.message_get(async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str prefer: The server behavior requested by the client. Use 'respond-async' to execute the request asynchronously. If the request is executed asynchronously, 'Preference-applied: respond-async' is returned in the response.
        :param str x_net_suite_idempotency_key: A user-defined unique idempotency key that is applied to every asynchronous requests to ensure that the request is executed only once. Only one request can be executed with every unique idempotency key. Use UUID in string format as defined by RFC 4122. If the request is executed synchronously, this value is ignored.
        :param str q: The search query that is used to filter results.
        :param int limit: The limit used to specify the number of results on a single page.
        :param int offset: The offset used for selecting a specific page of results.
        :return: MessageCollection
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = True
        if kwargs.get('async_req'):
            return self.message_get_with_http_info(**kwargs)  # noqa: E501
        else:
            (data) = self.message_get_with_http_info(**kwargs)  # noqa: E501
            return data

    def message_get_with_http_info(self, **kwargs):  # noqa: E501
        """Get list of records.  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.message_get_with_http_info(async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str prefer: The server behavior requested by the client. Use 'respond-async' to execute the request asynchronously. If the request is executed asynchronously, 'Preference-applied: respond-async' is returned in the response.
        :param str x_net_suite_idempotency_key: A user-defined unique idempotency key that is applied to every asynchronous requests to ensure that the request is executed only once. Only one request can be executed with every unique idempotency key. Use UUID in string format as defined by RFC 4122. If the request is executed synchronously, this value is ignored.
        :param str q: The search query that is used to filter results.
        :param int limit: The limit used to specify the number of results on a single page.
        :param int offset: The offset used for selecting a specific page of results.
        :return: MessageCollection
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ['prefer', 'x_net_suite_idempotency_key', 'q', 'limit', 'offset']  # noqa: E501
        all_params.append('async_req')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        params = locals()
        for key, val in six.iteritems(params['kwargs']):
            if key not in all_params:
                raise TypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method message_get" % key
                )
            params[key] = val
        del params['kwargs']

        collection_formats = {}

        path_params = {}

        query_params = []
        if 'q' in params:
            query_params.append(('q', params['q']))  # noqa: E501
        if 'limit' in params:
            query_params.append(('limit', params['limit']))  # noqa: E501
        if 'offset' in params:
            query_params.append(('offset', params['offset']))  # noqa: E501

        header_params = {}
        if 'prefer' in params:
            header_params['Prefer'] = params['prefer']  # noqa: E501
        if 'x_net_suite_idempotency_key' in params:
            header_params['X-NetSuite-Idempotency-Key'] = params['x_net_suite_idempotency_key']  # noqa: E501

        form_params = []
        local_var_files = {}

        body_params = None
        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.select_header_accept(
            ['application/vnd.oracle.resource+json; type=collection', 'application/vnd.oracle.resource+json; type=error'])  # noqa: E501

        # Authentication setting
        auth_settings = ['oAuth2ClientCredentials']  # noqa: E501

        return self.api_client.call_api(
            '/message', 'GET',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type='MessageCollection',  # noqa: E501
            auth_settings=auth_settings,
            async_req=params.get('async_req'),
            _return_http_data_only=params.get('_return_http_data_only'),
            _preload_content=params.get('_preload_content', True),
            _request_timeout=params.get('_request_timeout'),
            collection_formats=collection_formats)

    def message_id_delete(self, id, **kwargs):  # noqa: E501
        """Remove record.  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.message_id_delete(id, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param int id: Internal identifier. (required)
        :param str prefer: The server behavior requested by the client. Use 'respond-async' to execute the request asynchronously. If the request is executed asynchronously, 'Preference-applied: respond-async' is returned in the response.
        :param str x_net_suite_idempotency_key: A user-defined unique idempotency key that is applied to every asynchronous requests to ensure that the request is executed only once. Only one request can be executed with every unique idempotency key. Use UUID in string format as defined by RFC 4122. If the request is executed synchronously, this value is ignored.
        :return: None
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = True
        if kwargs.get('async_req'):
            return self.message_id_delete_with_http_info(id, **kwargs)  # noqa: E501
        else:
            (data) = self.message_id_delete_with_http_info(id, **kwargs)  # noqa: E501
            return data

    def message_id_delete_with_http_info(self, id, **kwargs):  # noqa: E501
        """Remove record.  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.message_id_delete_with_http_info(id, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param int id: Internal identifier. (required)
        :param str prefer: The server behavior requested by the client. Use 'respond-async' to execute the request asynchronously. If the request is executed asynchronously, 'Preference-applied: respond-async' is returned in the response.
        :param str x_net_suite_idempotency_key: A user-defined unique idempotency key that is applied to every asynchronous requests to ensure that the request is executed only once. Only one request can be executed with every unique idempotency key. Use UUID in string format as defined by RFC 4122. If the request is executed synchronously, this value is ignored.
        :return: None
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ['id', 'prefer', 'x_net_suite_idempotency_key']  # noqa: E501
        all_params.append('async_req')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        params = locals()
        for key, val in six.iteritems(params['kwargs']):
            if key not in all_params:
                raise TypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method message_id_delete" % key
                )
            params[key] = val
        del params['kwargs']
        # verify the required parameter 'id' is set
        if ('id' not in params or
                params['id'] is None):
            raise ValueError("Missing the required parameter `id` when calling `message_id_delete`")  # noqa: E501

        collection_formats = {}

        path_params = {}
        if 'id' in params:
            path_params['id'] = params['id']  # noqa: E501

        query_params = []

        header_params = {}
        if 'prefer' in params:
            header_params['Prefer'] = params['prefer']  # noqa: E501
        if 'x_net_suite_idempotency_key' in params:
            header_params['X-NetSuite-Idempotency-Key'] = params['x_net_suite_idempotency_key']  # noqa: E501

        form_params = []
        local_var_files = {}

        body_params = None
        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.select_header_accept(
            ['application/vnd.oracle.resource+json; type=error'])  # noqa: E501

        # Authentication setting
        auth_settings = ['oAuth2ClientCredentials']  # noqa: E501

        return self.api_client.call_api(
            '/message/{id}', 'DELETE',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type=None,  # noqa: E501
            auth_settings=auth_settings,
            async_req=params.get('async_req'),
            _return_http_data_only=params.get('_return_http_data_only'),
            _preload_content=params.get('_preload_content', True),
            _request_timeout=params.get('_request_timeout'),
            collection_formats=collection_formats)

    def message_id_get(self, id, **kwargs):  # noqa: E501
        """Get record.  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.message_id_get(id, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param int id: Internal identifier. (required)
        :param str prefer: The server behavior requested by the client. Use 'respond-async' to execute the request asynchronously. If the request is executed asynchronously, 'Preference-applied: respond-async' is returned in the response.
        :param str x_net_suite_idempotency_key: A user-defined unique idempotency key that is applied to every asynchronous requests to ensure that the request is executed only once. Only one request can be executed with every unique idempotency key. Use UUID in string format as defined by RFC 4122. If the request is executed synchronously, this value is ignored.
        :param bool expand_sub_resources: Set to 'true' to automatically expand all sublists, sublist lines, and subrecords on this record.
        :param bool simple_enum_format: Set to true to return enumeration values in a format that only shows the internal ID value.
        :param str fields: The names of the fields and sublists on the record. Only the selected fields and sublists will be returned in the response.
        :return: Message
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = True
        if kwargs.get('async_req'):
            return self.message_id_get_with_http_info(id, **kwargs)  # noqa: E501
        else:
            (data) = self.message_id_get_with_http_info(id, **kwargs)  # noqa: E501
            return data

    def message_id_get_with_http_info(self, id, **kwargs):  # noqa: E501
        """Get record.  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.message_id_get_with_http_info(id, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param int id: Internal identifier. (required)
        :param str prefer: The server behavior requested by the client. Use 'respond-async' to execute the request asynchronously. If the request is executed asynchronously, 'Preference-applied: respond-async' is returned in the response.
        :param str x_net_suite_idempotency_key: A user-defined unique idempotency key that is applied to every asynchronous requests to ensure that the request is executed only once. Only one request can be executed with every unique idempotency key. Use UUID in string format as defined by RFC 4122. If the request is executed synchronously, this value is ignored.
        :param bool expand_sub_resources: Set to 'true' to automatically expand all sublists, sublist lines, and subrecords on this record.
        :param bool simple_enum_format: Set to true to return enumeration values in a format that only shows the internal ID value.
        :param str fields: The names of the fields and sublists on the record. Only the selected fields and sublists will be returned in the response.
        :return: Message
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ['id', 'prefer', 'x_net_suite_idempotency_key', 'expand_sub_resources', 'simple_enum_format', 'fields']  # noqa: E501
        all_params.append('async_req')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        params = locals()
        for key, val in six.iteritems(params['kwargs']):
            if key not in all_params:
                raise TypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method message_id_get" % key
                )
            params[key] = val
        del params['kwargs']
        # verify the required parameter 'id' is set
        if ('id' not in params or
                params['id'] is None):
            raise ValueError("Missing the required parameter `id` when calling `message_id_get`")  # noqa: E501

        collection_formats = {}

        path_params = {}
        if 'id' in params:
            path_params['id'] = params['id']  # noqa: E501

        query_params = []
        if 'expand_sub_resources' in params:
            query_params.append(('expandSubResources', params['expand_sub_resources']))  # noqa: E501
        if 'simple_enum_format' in params:
            query_params.append(('simpleEnumFormat', params['simple_enum_format']))  # noqa: E501
        if 'fields' in params:
            query_params.append(('fields', params['fields']))  # noqa: E501

        header_params = {}
        if 'prefer' in params:
            header_params['Prefer'] = params['prefer']  # noqa: E501
        if 'x_net_suite_idempotency_key' in params:
            header_params['X-NetSuite-Idempotency-Key'] = params['x_net_suite_idempotency_key']  # noqa: E501

        form_params = []
        local_var_files = {}

        body_params = None
        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.select_header_accept(
            ['application/vnd.oracle.resource+json; type=singular', 'application/vnd.oracle.resource+json; type=error'])  # noqa: E501

        # Authentication setting
        auth_settings = ['oAuth2ClientCredentials']  # noqa: E501

        return self.api_client.call_api(
            '/message/{id}', 'GET',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type='Message',  # noqa: E501
            auth_settings=auth_settings,
            async_req=params.get('async_req'),
            _return_http_data_only=params.get('_return_http_data_only'),
            _preload_content=params.get('_preload_content', True),
            _request_timeout=params.get('_request_timeout'),
            collection_formats=collection_formats)

    def message_id_patch(self, body, id, **kwargs):  # noqa: E501
        """Update record.  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.message_id_patch(body, id, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param Message body: Request body. (required)
        :param int id: Internal identifier. (required)
        :param str prefer: The server behavior requested by the client. Use 'respond-async' to execute the request asynchronously. If the request is executed asynchronously, 'Preference-applied: respond-async' is returned in the response.
        :param str x_net_suite_idempotency_key: A user-defined unique idempotency key that is applied to every asynchronous requests to ensure that the request is executed only once. Only one request can be executed with every unique idempotency key. Use UUID in string format as defined by RFC 4122. If the request is executed synchronously, this value is ignored.
        :param str x_net_suite_property_name_validation: Sets the strictness of property name validation.
        :param str x_net_suite_property_value_validation: Sets the strictness of property value validation.
        :param str replace: The names of sublists on this record. All sublist lines will be replaced with lines specified in the request. The names are delimited by comma.
        :param bool replace_selected_fields: If set to 'true', all fields that should be deleted in the update request, including body fields, must be included in the 'replace' query parameter.
        :return: None
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = True
        if kwargs.get('async_req'):
            return self.message_id_patch_with_http_info(body, id, **kwargs)  # noqa: E501
        else:
            (data) = self.message_id_patch_with_http_info(body, id, **kwargs)  # noqa: E501
            return data

    def message_id_patch_with_http_info(self, body, id, **kwargs):  # noqa: E501
        """Update record.  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.message_id_patch_with_http_info(body, id, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param Message body: Request body. (required)
        :param int id: Internal identifier. (required)
        :param str prefer: The server behavior requested by the client. Use 'respond-async' to execute the request asynchronously. If the request is executed asynchronously, 'Preference-applied: respond-async' is returned in the response.
        :param str x_net_suite_idempotency_key: A user-defined unique idempotency key that is applied to every asynchronous requests to ensure that the request is executed only once. Only one request can be executed with every unique idempotency key. Use UUID in string format as defined by RFC 4122. If the request is executed synchronously, this value is ignored.
        :param str x_net_suite_property_name_validation: Sets the strictness of property name validation.
        :param str x_net_suite_property_value_validation: Sets the strictness of property value validation.
        :param str replace: The names of sublists on this record. All sublist lines will be replaced with lines specified in the request. The names are delimited by comma.
        :param bool replace_selected_fields: If set to 'true', all fields that should be deleted in the update request, including body fields, must be included in the 'replace' query parameter.
        :return: None
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ['body', 'id', 'prefer', 'x_net_suite_idempotency_key', 'x_net_suite_property_name_validation', 'x_net_suite_property_value_validation', 'replace', 'replace_selected_fields']  # noqa: E501
        all_params.append('async_req')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        params = locals()
        for key, val in six.iteritems(params['kwargs']):
            if key not in all_params:
                raise TypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method message_id_patch" % key
                )
            params[key] = val
        del params['kwargs']
        # verify the required parameter 'body' is set
        if ('body' not in params or
                params['body'] is None):
            raise ValueError("Missing the required parameter `body` when calling `message_id_patch`")  # noqa: E501
        # verify the required parameter 'id' is set
        if ('id' not in params or
                params['id'] is None):
            raise ValueError("Missing the required parameter `id` when calling `message_id_patch`")  # noqa: E501

        collection_formats = {}

        path_params = {}
        if 'id' in params:
            path_params['id'] = params['id']  # noqa: E501

        query_params = []
        if 'replace' in params:
            query_params.append(('replace', params['replace']))  # noqa: E501
        if 'replace_selected_fields' in params:
            query_params.append(('replaceSelectedFields', params['replace_selected_fields']))  # noqa: E501

        header_params = {}
        if 'prefer' in params:
            header_params['Prefer'] = params['prefer']  # noqa: E501
        if 'x_net_suite_idempotency_key' in params:
            header_params['X-NetSuite-Idempotency-Key'] = params['x_net_suite_idempotency_key']  # noqa: E501
        if 'x_net_suite_property_name_validation' in params:
            header_params['X-NetSuite-PropertyNameValidation'] = params['x_net_suite_property_name_validation']  # noqa: E501
        if 'x_net_suite_property_value_validation' in params:
            header_params['X-NetSuite-PropertyValueValidation'] = params['x_net_suite_property_value_validation']  # noqa: E501

        form_params = []
        local_var_files = {}

        body_params = None
        if 'body' in params:
            body_params = params['body']
        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.select_header_accept(
            ['application/vnd.oracle.resource+json; type=error'])  # noqa: E501

        # HTTP header `Content-Type`
        header_params['Content-Type'] = self.api_client.select_header_content_type(  # noqa: E501
            ['application/vnd.oracle.resource+json; type=singular'])  # noqa: E501

        # Authentication setting
        auth_settings = ['oAuth2ClientCredentials']  # noqa: E501

        return self.api_client.call_api(
            '/message/{id}', 'PATCH',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type=None,  # noqa: E501
            auth_settings=auth_settings,
            async_req=params.get('async_req'),
            _return_http_data_only=params.get('_return_http_data_only'),
            _preload_content=params.get('_preload_content', True),
            _request_timeout=params.get('_request_timeout'),
            collection_formats=collection_formats)

    def message_id_put(self, body, id, **kwargs):  # noqa: E501
        """Insert or update record.  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.message_id_put(body, id, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param Message body: Request body. (required)
        :param str id: External identifier. (required)
        :param str prefer: The server behavior requested by the client. Use 'respond-async' to execute the request asynchronously. If the request is executed asynchronously, 'Preference-applied: respond-async' is returned in the response.
        :param str x_net_suite_idempotency_key: A user-defined unique idempotency key that is applied to every asynchronous requests to ensure that the request is executed only once. Only one request can be executed with every unique idempotency key. Use UUID in string format as defined by RFC 4122. If the request is executed synchronously, this value is ignored.
        :param str x_net_suite_property_name_validation: Sets the strictness of property name validation.
        :param str x_net_suite_property_value_validation: Sets the strictness of property value validation.
        :param str replace: The names of sublists on this record. All sublist lines will be replaced with lines specified in the request. The names are delimited by comma.
        :param bool replace_selected_fields: If set to 'true', all fields that should be deleted in the update request, including body fields, must be included in the 'replace' query parameter.
        :return: None
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = True
        if kwargs.get('async_req'):
            return self.message_id_put_with_http_info(body, id, **kwargs)  # noqa: E501
        else:
            (data) = self.message_id_put_with_http_info(body, id, **kwargs)  # noqa: E501
            return data

    def message_id_put_with_http_info(self, body, id, **kwargs):  # noqa: E501
        """Insert or update record.  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.message_id_put_with_http_info(body, id, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param Message body: Request body. (required)
        :param str id: External identifier. (required)
        :param str prefer: The server behavior requested by the client. Use 'respond-async' to execute the request asynchronously. If the request is executed asynchronously, 'Preference-applied: respond-async' is returned in the response.
        :param str x_net_suite_idempotency_key: A user-defined unique idempotency key that is applied to every asynchronous requests to ensure that the request is executed only once. Only one request can be executed with every unique idempotency key. Use UUID in string format as defined by RFC 4122. If the request is executed synchronously, this value is ignored.
        :param str x_net_suite_property_name_validation: Sets the strictness of property name validation.
        :param str x_net_suite_property_value_validation: Sets the strictness of property value validation.
        :param str replace: The names of sublists on this record. All sublist lines will be replaced with lines specified in the request. The names are delimited by comma.
        :param bool replace_selected_fields: If set to 'true', all fields that should be deleted in the update request, including body fields, must be included in the 'replace' query parameter.
        :return: None
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ['body', 'id', 'prefer', 'x_net_suite_idempotency_key', 'x_net_suite_property_name_validation', 'x_net_suite_property_value_validation', 'replace', 'replace_selected_fields']  # noqa: E501
        all_params.append('async_req')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        params = locals()
        for key, val in six.iteritems(params['kwargs']):
            if key not in all_params:
                raise TypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method message_id_put" % key
                )
            params[key] = val
        del params['kwargs']
        # verify the required parameter 'body' is set
        if ('body' not in params or
                params['body'] is None):
            raise ValueError("Missing the required parameter `body` when calling `message_id_put`")  # noqa: E501
        # verify the required parameter 'id' is set
        if ('id' not in params or
                params['id'] is None):
            raise ValueError("Missing the required parameter `id` when calling `message_id_put`")  # noqa: E501

        collection_formats = {}

        path_params = {}
        if 'id' in params:
            path_params['id'] = params['id']  # noqa: E501

        query_params = []
        if 'replace' in params:
            query_params.append(('replace', params['replace']))  # noqa: E501
        if 'replace_selected_fields' in params:
            query_params.append(('replaceSelectedFields', params['replace_selected_fields']))  # noqa: E501

        header_params = {}
        if 'prefer' in params:
            header_params['Prefer'] = params['prefer']  # noqa: E501
        if 'x_net_suite_idempotency_key' in params:
            header_params['X-NetSuite-Idempotency-Key'] = params['x_net_suite_idempotency_key']  # noqa: E501
        if 'x_net_suite_property_name_validation' in params:
            header_params['X-NetSuite-PropertyNameValidation'] = params['x_net_suite_property_name_validation']  # noqa: E501
        if 'x_net_suite_property_value_validation' in params:
            header_params['X-NetSuite-PropertyValueValidation'] = params['x_net_suite_property_value_validation']  # noqa: E501

        form_params = []
        local_var_files = {}

        body_params = None
        if 'body' in params:
            body_params = params['body']
        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.select_header_accept(
            ['application/vnd.oracle.resource+json; type=error'])  # noqa: E501

        # HTTP header `Content-Type`
        header_params['Content-Type'] = self.api_client.select_header_content_type(  # noqa: E501
            ['application/vnd.oracle.resource+json; type=singular'])  # noqa: E501

        # Authentication setting
        auth_settings = ['oAuth2ClientCredentials']  # noqa: E501

        return self.api_client.call_api(
            '/message/{id}', 'PUT',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type=None,  # noqa: E501
            auth_settings=auth_settings,
            async_req=params.get('async_req'),
            _return_http_data_only=params.get('_return_http_data_only'),
            _preload_content=params.get('_preload_content', True),
            _request_timeout=params.get('_request_timeout'),
            collection_formats=collection_formats)

    def message_post(self, body, **kwargs):  # noqa: E501
        """Insert record.  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.message_post(body, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param Message body: Request body. (required)
        :param str prefer: The server behavior requested by the client. Use 'respond-async' to execute the request asynchronously. If the request is executed asynchronously, 'Preference-applied: respond-async' is returned in the response.
        :param str x_net_suite_idempotency_key: A user-defined unique idempotency key that is applied to every asynchronous requests to ensure that the request is executed only once. Only one request can be executed with every unique idempotency key. Use UUID in string format as defined by RFC 4122. If the request is executed synchronously, this value is ignored.
        :param str x_net_suite_property_name_validation: Sets the strictness of property name validation.
        :param str x_net_suite_property_value_validation: Sets the strictness of property value validation.
        :param str replace: The names of sublists on this record. All sublist lines will be replaced with lines specified in the request. The names are delimited by comma.
        :return: None
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = True
        if kwargs.get('async_req'):
            return self.message_post_with_http_info(body, **kwargs)  # noqa: E501
        else:
            (data) = self.message_post_with_http_info(body, **kwargs)  # noqa: E501
            return data

    def message_post_with_http_info(self, body, **kwargs):  # noqa: E501
        """Insert record.  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.message_post_with_http_info(body, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param Message body: Request body. (required)
        :param str prefer: The server behavior requested by the client. Use 'respond-async' to execute the request asynchronously. If the request is executed asynchronously, 'Preference-applied: respond-async' is returned in the response.
        :param str x_net_suite_idempotency_key: A user-defined unique idempotency key that is applied to every asynchronous requests to ensure that the request is executed only once. Only one request can be executed with every unique idempotency key. Use UUID in string format as defined by RFC 4122. If the request is executed synchronously, this value is ignored.
        :param str x_net_suite_property_name_validation: Sets the strictness of property name validation.
        :param str x_net_suite_property_value_validation: Sets the strictness of property value validation.
        :param str replace: The names of sublists on this record. All sublist lines will be replaced with lines specified in the request. The names are delimited by comma.
        :return: None
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ['body', 'prefer', 'x_net_suite_idempotency_key', 'x_net_suite_property_name_validation', 'x_net_suite_property_value_validation', 'replace']  # noqa: E501
        all_params.append('async_req')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        params = locals()
        for key, val in six.iteritems(params['kwargs']):
            if key not in all_params:
                raise TypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method message_post" % key
                )
            params[key] = val
        del params['kwargs']
        # verify the required parameter 'body' is set
        if ('body' not in params or
                params['body'] is None):
            raise ValueError("Missing the required parameter `body` when calling `message_post`")  # noqa: E501

        collection_formats = {}

        path_params = {}

        query_params = []
        if 'replace' in params:
            query_params.append(('replace', params['replace']))  # noqa: E501

        header_params = {}
        if 'prefer' in params:
            header_params['Prefer'] = params['prefer']  # noqa: E501
        if 'x_net_suite_idempotency_key' in params:
            header_params['X-NetSuite-Idempotency-Key'] = params['x_net_suite_idempotency_key']  # noqa: E501
        if 'x_net_suite_property_name_validation' in params:
            header_params['X-NetSuite-PropertyNameValidation'] = params['x_net_suite_property_name_validation']  # noqa: E501
        if 'x_net_suite_property_value_validation' in params:
            header_params['X-NetSuite-PropertyValueValidation'] = params['x_net_suite_property_value_validation']  # noqa: E501

        form_params = []
        local_var_files = {}

        body_params = None
        if 'body' in params:
            body_params = params['body']
        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.select_header_accept(
            ['application/vnd.oracle.resource+json; type=error'])  # noqa: E501

        # HTTP header `Content-Type`
        header_params['Content-Type'] = self.api_client.select_header_content_type(  # noqa: E501
            ['application/vnd.oracle.resource+json; type=singular'])  # noqa: E501

        # Authentication setting
        auth_settings = ['oAuth2ClientCredentials']  # noqa: E501

        return self.api_client.call_api(
            '/message', 'POST',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type=None,  # noqa: E501
            auth_settings=auth_settings,
            async_req=params.get('async_req'),
            _return_http_data_only=params.get('_return_http_data_only'),
            _preload_content=params.get('_preload_content', True),
            _request_timeout=params.get('_request_timeout'),
            collection_formats=collection_formats)
